# AI_AUTO_PLAY_MOD
#
# version 1.0a

from CvPythonExtensions import *
import CvUtil

gc = CyGlobalContext()
game = CyGame()

import PyCivSim

HOTKEY_AUTO = "Control-Shift-X"
filename = 'AIData.csv'

#-------------------------------------------------------------------------
# Event handlers
def EventHandlerRegister(evt_mgr, unused = None):
	CvUtil.pyPrint( __name__ + ": Event handler register OK." )
	
	AILogs = AILogger()
	
	return  { 
			"BeginGameTurn" 		:	AILogs.onBeginGameTurn,
			"victory"  		: 	AILogs.onVictory,	
			"kbdEvent" 		:	AILogs.onKbdEvent,
			}
			
HOTKEY_AUTO = PyCivSim.getHotkey("AI_TESTER_MOD", "RUNTEST", HOTKEY_AUTO)

class AILogger:
	def __init__(self) :
		self.TurnsToAuto = 100000
		self.numGames = 0
	
		# Top level is the player
		# 2nd level is the turn
		# 3rd level is the value
		self.AIScores = []
		self.AIPowers = []
		self.AIRanks = []
		self.AIVictories = []
		
		for i in range(gc.getMAX_CIV_PLAYERS()):
			self.AIScores.append([])
			self.AIPowers.append([])
			self.AIRanks.append([])
			self.AIVictories.append([])

	def onKbdEvent( self, argsList ) :
		'keypress handler'
#	   eventType,key,mx,my,px,py = argsList
#		( isKeyDownEvent, theKey, mx, my, px, py, 
#			( keyModifier, bShift, bCtrl, bAlt), Unused ) = argsList

		if PyCivSim.checkKeyInput( argsList, HOTKEY_AUTO ) :
			# Get it?  Shift ... control ... to the AI
			if( game.getAIAutoPlay() > 0 ) :
				self.TurnsToAuto = 0
				self.setAIAutoPlay()
			else :
				# Random big number
				self.TurnsToAuto = 100000
				self.setAIAutoPlay()
			return 1
			
	
	def onVictory(self, argsList):
		iTeam, iVictory = argsList
			
		for i in range(gc.getMAX_CIV_PLAYERS()):
			pPlayer = gc.getPlayer(i)
			if pPlayer.isEverAlive():
				if pPlayer.getTeam() == iTeam:
					self.AIVictories[i] = iVictory
				else:
					self.AIVictories[i] = (-1)
					
		self.logData()
		
		self.newGame(argsList)
		
		self.numGames += 1
		
		self.AIScores = []
		self.AIPowers = []
		self.AIRanks = []
		self.AIVictories = []
		
		for i in range(gc.getMAX_CIV_PLAYERS()):
			self.AIScores.append([])
			self.AIPowers.append([])
			self.AIRanks.append([])
			self.AIVictories.append([])
				
	
	def logData(self):
		try:
			f = open(filename, "r")
		except IOError:
			f = open(filename, "w")
			f.close()
			f = open(filename, "r")
		
		oldLineList = f.readlines()
		nextLine = 0
		
		newLineList = [""]
		
		if len(oldLineList) == 0:
			for i in range(gc.getMAX_CIV_PLAYERS()):
				lineStr = ""
				lineStr += "Player " + str(i) + ","
				lineStr += "Score,"
				lineStr += "Power,"
				lineStr += "Rank,"
				lineStr += "Victory Types,"
				newLineList[0] += lineStr
		
			nextLine = 1
		
		for i in range(gc.getMAX_CIV_PLAYERS()):
			if gc.getPlayer(i).isEverAlive():
				k = 0
				for j in range(max(len(self.AIScores[i]), len(self.AIPowers[i]), len(self.AIRanks[i]))):
					if self.AIScores[i][j] != [] and self.AIPowers[i][j] != [] and self.AIRanks[i][j] != []:
						k += 1
					
						lineStr = str(j) + ","
						lineStr += str(self.AIScores[i][j]) + ","
						lineStr += str(self.AIPowers[i][j]) + ","
						lineStr += str(self.AIRanks[i][j]) + ","

						if k != len(self.AIScores[i]):
							lineStr += ","						
					
						self.addLineToLoopList(nextLine, k, newLineList, lineStr)
				
				lineStr = str(self.AIVictories[i]) + ","
				self.addLineToLoopList(nextLine, k, newLineList, lineStr)
		
			if i == gc.getMAX_CIV_PLAYERS() - 1:
				nextLine = len(newLineList)
							
		f.close()
		
		f = open(filename, "w")
					
		# Annoyingly enough readlines() automatically does new lines as well, so I have to do these separately
		for line in oldLineList:
			f.write(line)
						
		for line in newLineList:
			f.write(line)
			f.write("\n")
			
		f.close()
						
	def addLineToLoopList(self, nextLine, j, newLineList, lineStr):
		try:
			newLineList[nextLine + j] += lineStr		
		except IndexError:
			newLineList.append("")
			self.addLineToLoopList(nextLine, j, newLineList, lineStr)
			
	def newGame( self, argsList ) :
		global game
		iTeam, iVictory = argsList
		
		iWorldSize = 0 # Doesn't matter - is set inside the function. Currently not supported
		iClimate = game.getSorenRandNum(gc.getNumClimateInfos(), "randomClimate")
		iSeaLevel = game.getSorenRandNum(gc.getNumSeaLevelInfos(), "randomSeaLevel")
		iGameSpeed = game.getSorenRandNum(gc.getNumGameSpeedInfos(), "randomGameSpeed")
		iNumPlayers = max(2, gc.getWorldInfo(CyMap().getWorldSize()).getDefaultPlayers() + game.getSorenRandNum(2, "randomPlayers") - 1)
		paiCivilizations = self.getRandomCivs(iNumPlayers)
		
		paiLeaderHeads = []
		for iCiv in paiCivilizations:
			civLeaderList = []
			for i in range(gc.getNumLeaderHeadInfos()):
				if gc.getCivilizationInfo(iCiv).isLeaders(i):
					civLeaderList.append(i)
			paiLeaderHeads.append(civLeaderList[game.getSorenRandNum(len(civLeaderList), "randomLeaderheads")])
		
		paiTeams = []
		for i in range(iNumPlayers):
			if i != gc.getBARBARIAN_TEAM():
				paiTeams.append(i)
		
		if ( game.getAIAutoPlay( ) > 0 ) :
			game.newGame(iWorldSize, iClimate, iSeaLevel, iGameSpeed, paiCivilizations, paiLeaderHeads, paiTeams)
			game = CyGame()	
			# Big number - want to auto untill end of the game
			self.TurnsToAuto = 10000
			self.setAIAutoPlay()
		else:
			game.newGame(iWorldSize, iClimate, iSeaLevel, iGameSpeed, paiCivilizations, paiLeaderHeads, paiTeams)
			game = CyGame()
		

	def setAIAutoPlay( self ) :
		game.setAIAutoPlay( self.TurnsToAuto )
		
	
	def getRandomCivs(self, iNumCivs):
		randCivList = []
			
		while len(randCivList) < iNumCivs:
			rand = game.getSorenRandNum(gc.getNumCivilizationInfos(), "randomCiv")
			
			if rand != gc.getBARBARIAN_PLAYER():
				if rand not in randCivList:
					randCivList.append(rand)
			
		return randCivList
			
	def onBeginGameTurn(self, argsList):
		iGameTurn = argsList[0]
					
		for i in range(gc.getMAX_CIV_PLAYERS()):
			if gc.getPlayer(i).isEverAlive():
				self.AIScores[i].append([])
				self.AIPowers[i].append([])
				self.AIRanks[i].append([])
				
				self.AIScores[i][iGameTurn] = CyGame().getPlayerScore(i)
				self.AIPowers[i][iGameTurn] = gc.getPlayer(i).getPower()
				self.AIRanks[i][iGameTurn] = CyGame().getPlayerRank(i)					
			
