# $Source: /cvsroot/civ4ccp/civ4ccp/Assets/Python/CUSTOM/DEMO/Attic/DEMO_DemolitionMod.py,v $: $Name: dev $
#	SimCutie's Demolition Mod for DEMO Example.
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#
#	This is mod that enables building demolition/ city raze by player.
#	To raze your city, choose a city and press "Control-Shift-X" hot key.
#	When city management screen is up, bulding can be demolished, instead.
#
#	The changes here-in are based mainly on Snaitf's  Demolition Mod
#	which is, in tern, based on TheDarkside's Building Demolition
#	Mod and Arkeide's City Demolition Mod with many of Snaitf's own changes 
#
#	Feel free to use this in your own mod, alter or redistribute this mod as  ##
#	long is is of a non-profit nature and SimCutie, Snaitf, TheDarsike, Arkeide
#	are  credited accordingly.
#
################################################################################
__author__ = "$Author: simcutie $"
__revision__ = "$Id: DEMO_DemolitionMod.py,v 1.1.2.1 2006/07/21 03:44:10 simcutie Exp $"
__version__ = "$Revision: 1.1.2.1 $"
__date__ = "$Date: 2006/07/21 03:44:10 $"

from CvPythonExtensions import *
import CvUtil

import PyCivSim
from PyCivSim import PopupStyles

gc = CyGlobalContext()
localText = CyTranslator()

HOTKEY_DEMOLISH = "Control-Shift-X"

#-------------------------------------------------------------------------
# Event handlers
def EventHandlerRegister(evt_mgr, unused = None):
	CvUtil.pyPrint( __name__ + ": Event handler register OK." )
	return { "kbdEvent" :	onKbdEvent }

#################### ON EVENTS ######################

HOTKEY_DEMOLISH = PyCivSim.getHotkey( "DEMOLITION_MOD", "DEMOLISH", HOTKEY_DEMOLISH)

def onKbdEvent( argsList):
	'keypress handler - return 1 if the event was consumed'

#	( isKeyDownEvent, theKey, mx, my, px, py, 
#		( keyModifier, bShift, bCtrl, bAlt), Unused ) = argsList
#	if( isKeyDownEvent and theKey == int(InputTypes.KB_X) and bCtrl ):

	pCity = CyInterface().getHeadSelectedCity()
	if ( not PyCivSim.checkKeyInput(argsList, HOTKEY_DEMOLISH )
		or not pCity or pCity.getOwner() != gc.getGame().getActivePlayer()):
		return 0
	if CyInterface().isCityScreenUp():
		demolishBuildingPopup(pCity)
	else :
		razeCityPopup(pCity)
	return 1

def canDemolishBuilding( pCity, eBuilding ) :
	"is building demolish applicable for this city? " 
	return ( eBuilding != gc.getInfoTypeForString("BUILDING_PALACE")
		and pCity.hasBuilding(eBuilding) 
		and not pCity.isFreeBuilding(eBuilding) 
		and not isWorldWonderClass( gc.getBuildingInfo(eBuilding).getBuildingClassType()))

def demolishBuildingPopup(pCity) :
	" Show demolish city building popup"

	# "Choose building to demolish" 
	popup = PyCivSim.createPopup(demolishBuildingApply, "TXT_KEY_DEMOLITION_BUILDING") 
	popup.createPullDown()

	numList = 0
	for iBuilding in range(gc.getNumBuildingInfos()) :
		if canDemolishBuilding( pCity, iBuilding ) :
			buildingInfo = gc.getBuildingInfo(iBuilding)
			gains = localText.getText("TXT_KEY_DEMOLITION_GOLD_AND_PRODUCTION", 
				((buildingInfo.getProductionCost() * 3 / 10 ), 
				 (buildingInfo.getProductionCost() / 10) ))	
			
			popup.addPullDownString( buildingInfo.getDescription() + " : " + gains, numList)
			numList += 1
	
	if numList > 0:
		PyCivSim.launchPopup(popup, PopupStyles.POPUPSTYLE_OKCANCEL)

def demolishBuildingApply(playerID, userData, popupReturn):
	" Demolish selected building and refund" 
	pCity = CyInterface().getHeadSelectedCity()
	iBuilding = popupReturn.getSelectedPullDownValue(0)

	if ( pCity and popupReturn.getButtonClicked() == 0):
		pCity.setHasRealBuilding( iBuilding, False)
		CyAudioGame().Play2DSound("AS2D_PILLAGE")
		buildingInfo = gc.getBuildingInfo(iBuilding)
		refundGold = ( buildingInfo.getProductionCost() * 3) / 10
		gc.getActivePlayer().setGold( gc.getActivePlayer().getGold() + refundGold )
		refundHammer = buildingInfo.getProductionCost() / 10
		pCity.setProduction(pCity.getProduction() + refundHammer )
	return

def razeCityPopup(pCity) :
	" Show raze city popup"

	refundGold = 0
	for iBuilding in range(gc.getNumBuildingInfos()) :
		if canDemolishBuilding( pCity, iBuilding ):
			refundGold += (gc.getBuildingInfo(iBuilding).getProductionCost() * 3) / 10

	head = localText.getText( "TXT_KEY_DEMOLITION_CITY", (pCity.getName(), refundGold ))
	popup = PyCivSim.createPopup( razeCityApply, head)
	PyCivSim.launchPopup(popup, PopupStyles.POPUPSTYLE_OKCANCEL)

def razeCityApply(playerID, userData, popupReturn):
	" Raze selected city and refund" 
	pCity = CyInterface().getHeadSelectedCity()
	if ( pCity and popupReturn.getButtonClicked() == 0):
		refund = 0
		for iBuilding in range(gc.getNumBuildingInfos()) :
			if canDemolishBuilding( pCity, iBuilding ):
				refund += (gc.getBuildingInfo(iBuilding).getProductionCost() * 3) / 10

		gc.getActivePlayer().setGold(gc.getActivePlayer().getGold() + refund )
		pCity.kill()
		CyAudioGame().Play2DSound("AS2D_CITYCAPTURE") 
	return 1

